@@ -216,15 +216,77 @@ pub fn read_floww_from_midi(path: &str) -> Result<Floww, ApresError>{
     }
 }
 
-#[derive(Serialize, Deserialize)]
+#[derive(Clone,PartialEq,Debug,Serialize, Deserialize)]
 pub enum FlowwPacket{
     Msg(String),
     Track(String),
     Point(Point),
 }
 
-pub fn encode(batch: &[FlowwPacket]) -> Vec<u8>{
-    bincode::serialize(batch).unwrap()
+pub trait IntoFlowwPacket{
+    fn into_packet(self) -> FlowwPacket;
+}
+
+impl IntoFlowwPacket for Point{
+    fn into_packet(self) -> FlowwPacket{
+        FlowwPacket::Point(self)
+    }
+}
+
+pub trait IntoFlowwPackets{
+    fn into_packets(self) -> Vec<FlowwPacket>;
+}
+
+impl<T: IntoFlowwPacket> IntoFlowwPackets for Vec<T>{
+    fn into_packets(self) -> Vec<FlowwPacket>{
+        self.into_iter().map(|i| i.into_packet()).collect::<Vec<_>>()
+    }
+}
+
+pub trait Encodable{
+    fn encode(&self) -> Vec<u8>;
+    fn encoded(self) -> Vec<u8>;
+}
+
+impl Encodable for Vec<FlowwPacket>{
+    fn encode(&self) -> Vec<u8>{
+        bincode::serialize(self).unwrap()
+    }
+
+    fn encoded(self) -> Vec<u8>{
+        self.encode()
+    }
+}
+
+pub trait DecodeIntoFlowwPackets{
+    fn decode(&self) -> Option<Vec<FlowwPacket>>;
+    fn decoded(self) -> Option<Vec<FlowwPacket>>;
+}
+
+impl DecodeIntoFlowwPackets for Vec<u8>{
+    fn decode(&self) -> Option<Vec<FlowwPacket>>{
+        match bincode::deserialize(self){
+            Ok(res) => Some(res),
+            Err(_) => None,
+        }
+    }
+
+    fn decoded(self) -> Option<Vec<FlowwPacket>>{
+        self.decode()
+    }
+}
+
+impl DecodeIntoFlowwPackets for &[u8]{
+    fn decode(&self) -> Option<Vec<FlowwPacket>>{
+        match bincode::deserialize(self){
+            Ok(res) => Some(res),
+            Err(_) => None,
+        }
+    }
+
+    fn decoded(self) -> Option<Vec<FlowwPacket>>{
+        self.decode()
+    }
 }
 
 #[derive(Clone,Copy,Default)]
@@ -241,7 +303,7 @@ impl FlowwDecoder{
 
     pub fn decode(&mut self, flowws: &mut Vec<Floww>, map: &HashMap<String, usize>, data: &[u8]) -> Vec<String>{
         let mut messages = Vec::new();
-        let batch: Vec<FlowwPacket> = if let Ok(res) = bincode::deserialize(data) { res }
+        let batch: Vec<FlowwPacket> = if let Some(res) = data.decoded() { res }
         else { return messages; };
         for packet in batch{
             match packet{
@@ -294,7 +356,7 @@ mod tests {
             FlowwPacket::Point((0, 0.0, 0.75, 1.0)),
         ];
 
-        let stream = encode(&a);
+        let stream = a.encoded();
         let mut decoder = FlowwDecoder::new();
         let messages = decoder.decode(&mut tracks, &map, &stream);
         assert_eq!(messages, vec!["beat".to_string()]);
@@ -307,7 +369,7 @@ mod tests {
             FlowwPacket::Track("crash".to_string()),
             FlowwPacket::Point((0, 0.0, 1.0, 1.0)),
         ];
-        let messages = decoder.decode(&mut tracks, &map, &encode(&b));
+        let messages = decoder.decode(&mut tracks, &map, &b.encoded());
         assert_eq!(messages, Vec::<String>::new());
         assert_eq!(tracks[0], vec![(0, 0.0, 0.25, 1.0), (0, 0.0, 0.75, 1.0), (0, 0.0, 1.0, 1.0)]);
         assert_eq!(tracks[2], vec![(0, 0.0, 1.0, 1.0)]);
@@ -332,5 +394,14 @@ mod tests {
         let h = g.scaled(2.0);
         assert_eq!(h, vec![(1, 0.0, 0.0, 0.0), (2, 1.0, 0.0, 0.0),
                             (0, 2.0, 0.0, 0.0), (3, 4.0, 0.0, 0.0)]);
+        let i = h.into_packets();
+        assert_eq!(i, vec![
+            FlowwPacket::Point((1, 0.0, 0.0, 0.0)),
+            FlowwPacket::Point((2, 1.0, 0.0, 0.0)),
+            FlowwPacket::Point((0, 2.0, 0.0, 0.0)),
+            FlowwPacket::Point((3, 4.0, 0.0, 0.0)),
+        ]);
+        let j = i.encode().decoded().unwrap();
+        assert_eq!(i, j);
     }
 }
